/*
 *  Multi Dictionary Lookup
 *  Website: https://addons.mozilla.org/en-US/firefox/addon/multi-dictionary-lookup/
 *  Source:  https://github.com/nohup/multidict_lookup
 *
 *  Copyright (c) 2012+ Nohup Technologies LLC
 *  Licensed under the Mozilla Public License, version 1.1
 */

/* Prevent Javascript namepsace pollution */
window.multidict_lookup = {
	clientX:0,
	clientY:0,
	pageX:0,
	pageY:0,
	prefs:null,
	dict:null,
	clickCount:0,
	aLinkTextNode:null,
	selection:'',
	/* List of dictionaries supported */
	/*
	 * 'menuitem-id': {
		 * pref: name of the preference element
		 * image: resource url for the icon
		 * title: title for attribution
		 * selector: selector api selctor to be used in querySelector
	 * }
	 * */
	"dictionaries": {
		"context-tfd": {
			"pref": "tfdURL",
			"image": "resource://multidict/tfd-icon.png",
			"title": "The Free Dictionary",
			"selector": "#MainTxt>table>tbody>tr:first-child>td:first-child"
		},
		"context-ud": {
			"pref": "udURL",
			"image": "resource://multidict/ud-icon.png",
			"title": "Urban Dictionary",
			"selector": "#entries>tbody>tr:nth-child(2)>td:nth-child(2)>div.definition"
		},
		"context-beed": {
			"pref": "beedURL",
			"image": "resource://multidict/beed-icon.png",
			"title": "Bee Dictionary",
			"selector": "#WordsDiv>ul>li:first-child"
		}
	},

	/**
	 * Initialize the Multi Dictionary plugin when the browser page has loaded
	 *
	 * @return void
	 **/
	onLoad: function() {
		this.initialized = true;

		/* load localization strings for the dictionary plugin */
		this.stringsBundle = document.getElementById("multidict_lookup-strings"); /*document here is the XUL document*/

		/* define what is allowed or not in the dictionary plugin pop-up */
		var tfdbrowser = document.getElementById("tfd-browser");
		if (tfdbrowser) {
			tfdbrowser.style.height = "0px";
			tfdbrowser.webNavigation.allowAuth = true;
			tfdbrowser.webNavigation.allowImages = true;
			tfdbrowser.webNavigation.allowJavascript = false;
			tfdbrowser.webNavigation.allowMetaRedirects = false;
			tfdbrowser.webNavigation.allowPlugins = false;
			tfdbrowser.webNavigation.allowSubframes = false;
			tfdbrowser.webNavigation.allowWindowControl = false;

			/* notify the dictionary plugin when the page being browsed has finished loading */
			tfdbrowser.addEventListener("DOMContentLoaded", window.multidict_lookup.onDefinitionLoad, true);

			/* load the dictionary plugin preferences */
			window.multidict_lookup.prefs = Components.classes["@mozilla.org/preferences-service;1"]
         .getService(Components.interfaces.nsIPrefService)
         .getBranch("extensions.multidict_lookup.");
			window.multidict_lookup.prefs.QueryInterface(Components.interfaces.nsIPrefBranch2);
		}

		/* notify the dictionary plugin when the mouse right-click brings up the browser context menu */
		var contextMenu = document.getElementById("contentAreaContextMenu");
		if(contextMenu){
			contextMenu.addEventListener("popupshowing", window.multidict_lookup.showFirefoxContextMenu, false);
		}

		/* track the mouse position every time the user clicks on the page */
		document.addEventListener("click",window.multidict_lookup.onClick);

		/* reset the plugin pop-up window at the top-left of the screen */
		window.multidict_lookup.clientX = 0;
		window.multidict_lookup.clientY = 0;

		/*Create a text node to show "Powered by" link*/
		window.multidict_lookup.aLinkTextNode = document.createTextNode('');
	},

	/**
	 * Verify if the mouse click is a right-click.
	 * If so, get ready to position the dictionary plugin pop-up window just below it.
	 *
	 * @return void
	 **/
	onClick: function(e) {
		if(e.button == 2) {
			/*on mouse right click, determine coordinates*/
			window.multidict_lookup.clientX = e.pageX;
			window.multidict_lookup.clientY = e.pageY + 18;
			//window.multidict_lookup.jsdump('click count '+ window.multidict_lookup.clickCount +' clientX '+ e.clientX +' pageY '+e.pageY);
		}
	},

	/**
	 * Verify if a string has been highlighted on the page.
	 * If so, get ready to position the dictionary plugin pop-up window just below it.
	 *
	 * @return void
	 **/
	onMouseUp: function(e) {
		/*This one will happen on selection*/
		if (window.multidict_lookup.clickCount == 0 && window.multidict_lookup.isTextSelected()) {
			window.multidict_lookup.clientX = e.pageX;
			window.multidict_lookup.clientY = e.pageY;
		}
		//window.multidict_lookup.jsdump('click count '+ window.multidict_lookup.clickCount +' clientX '+ e.clientX +' pageY '+e.pageY);
	},

	/**
	 * Called to determine if a string has been highlighted on the page.
	 *
	 * @return string
	 **/
	isTextSelected: function() {
		var textSelected = false;
		if (gContextMenu && gContextMenu.isContentSelected) {
			window.multidict_lookup.selection = document.commandDispatcher.focusedWindow.getSelection();
			window.multidict_lookup.selection = window.multidict_lookup.selection.toString().replace(/(\n|\r|\t)+/g, " ").trim();
			textSelected = (window.multidict_lookup.selection != "");
		}
		return textSelected;
	},

	/**
	 * Decide how to show the browser context menu entries for the dictionary plugin
	 * based on whether text has been highlighted.
	 *
	 * @return void
	 **/
	showFirefoxContextMenu: function(aEvent){
		var w; /* full string that has been highlighted for lookup */
		var truncated_w; /* first 15 chars of string that has been highlighted for lookup */

		var key;
		if (window.multidict_lookup.isTextSelected() && (w=window.multidict_lookup.getWordToLook()) != null) {

			/* if the string to lookup exceeds 15 chars, truncate it and add an ellipsis at the end */
			/* to prevent the browser context menu from stretching to an absurd width */
			if ( w.length > 15 ) {
				truncated_w = w.substring(0,14) + "...";
				w = truncated_w;
			}

			/* for each of the supported dictionaries, show the context menu as '<icon> Define "<term> [...]"' */
			for (key in window.multidict_lookup["dictionaries"]) {

			/* use localized strings */
				document.getElementById(key).setAttribute('label', stringsBundle.getString('multidict_lookup.contextMenu.labelDefine') +
					                                                   	" \"" + w + "\"");
				//document.getElementById(key).setAttribute('label',"Define \"" + w + "\"");
				document.getElementById(key).setAttribute('image',window.multidict_lookup["dictionaries"][key].image);
			}
		}

		/* if no text has been selected, don't show these context menu entries */
		for (key in window.multidict_lookup["dictionaries"]) {
			document.getElementById(key).hidden = !gContextMenu.isTextSelected;
		}
	},

	/**
	 * Return the string that has been highlighted to lookup in a dictionary.
	 *
	 * @return string
	 **/
	getWordToLook:function(){
		var sel = window.multidict_lookup.selection;
		/*
		var w = null;
		if(sel.length > 0){
			w = sel.split(' ',1)[0];
		}
		return w;
		*/
		return sel;
	},

	/**
	 * Determine which dictionary has been selected from the browser context menu.
	 * Form the URL needed to look up the highlighted text in the chosen dictionary.
	 * Request the URL and then show the dictionary plugin pop-up with loading animation.
	 *
	 * @return void
	 **/
	onMenuItemCommand: function(e) {
		window.multidict_lookup.dict = e.target.id;
		var url = '';
		/* Get the dictionary URL from preferences and append the string to lookup */
		url = window.multidict_lookup.prefs.getCharPref(window.multidict_lookup["dictionaries"][window.multidict_lookup.dict].pref);
		url += window.multidict_lookup.getWordToLook();

		/* Request the URL from the dictionary's website */
		var tfdbrowser = document.getElementById("tfd-browser");
		tfdbrowser.webNavigation.loadURI(url,0, null, null, null);

		/* Display the dictionary plugin pop-up window with a waiting animation */
		window.multidict_lookup.displayPopup(url);
	},

	/**
	 * Construct the dictionary plugin pop-up window.
	 *
	 * @return void
	 **/
	displayPopup: function(url) {

			/* define an outer container that will hold everything inside it */
		var outerDiv=content.document.getElementById('tfd');

		/* define an inner container to hold the definition retrieved from the dictionary */
		var definitionDiv=content.document.getElementById('tfd-lookup');

		/* define another inner container to hold the attribution to the dictionary */
		var attribution = null;

		/* create the outer div if it doesn't already exist */
		if(!definitionDiv){

			outerDiv=content.document.createElement('div');
			outerDiv.id="tfd";

			/* create the inner container for the word definition */
			definitionDiv= content.document.createElement('div');
			definitionDiv.id='tfd-lookup';

			/* create the attribution container */
			attribution = content.document.createElement('span');
			attribution.id="attribution";

			/* use localised strings */
			var power = content.document.createTextNode(stringsBundle.getString('multidict_lookup.popupWindow.labelPoweredBy') + " ");
			//var power = content.document.createTextNode('Powered by ');
			attribution.appendChild(power);

			/* create a link that holds the URL actually queried by the dictionary plugin */
			var aElement = content.document.createElement('a');
			aElement.id="attribution-link";
			aElement.target = '_blank';
			//window.multidict_lookup.aLinkTextNode = content.document.createTextNode('');
			//aElement.appendChild(window.multidict_lookup.aLinkTextNode);
			attribution.appendChild(aElement);

			/* attach the stylesheet used to style the elements above */
			var styl = content.document.createElement('link');
			styl.href='resource://multidict/skin.css';
			styl.rel='stylesheet';
			styl.type='text/css';

			outerDiv.appendChild(definitionDiv);
			outerDiv.appendChild(attribution);
			content.document.documentElement.appendChild(outerDiv);
			content.document.documentElement.appendChild(styl);
		};

		/* if the dictionary plugin pop-up has been shown before, clear it */
		while (definitionDiv.firstChild) {
			definitionDiv.removeChild(definitionDiv.firstChild);
		}

		/* show an animation while waiting for the dictionary URL to load */
		var loadingDiv= content.document.createElement('div');
		loadingDiv.id='tfd-loading';
		definitionDiv.appendChild(loadingDiv);

		/* show the correct dictionary icon in the attribution container */
		var attributionA = outerDiv.querySelector('#attribution-link');
		attributionA.href = url;

		/* use localised strings */
		attributionA.title = stringsBundle.getString('multidict_lookup.popupWindow.labelLookup') +
															' "' + window.multidict_lookup.getWordToLook() + '"';
		//attributionA.title = 'Lookup "'+ window.multidict_lookup.getWordToLook() + '"';
		/*If aLinkTextNode has a parent, remove it from the parent tree*/
		if (window.multidict_lookup.aLinkTextNode.parentNode) {
			window.multidict_lookup.aLinkTextNode.parentNode.removeChild(window.multidict_lookup.aLinkTextNode);
		}
		window.multidict_lookup.aLinkTextNode.textContent = window.multidict_lookup["dictionaries"][window.multidict_lookup.dict].title;
		attributionA.appendChild(window.multidict_lookup.aLinkTextNode);

		/* position the dictionary icon to the left of the attribution link */
		attribution = outerDiv.querySelector('#attribution');
		attribution.style.backgroundImage = "url("+ window.multidict_lookup["dictionaries"][window.multidict_lookup.dict].image +")";
		attribution.style.backgroundPosition = "left top";
		attribution.style.backgroundRepeat = "no-repeat";

		/* attempt to reset the scrollbar inside the dictionary plugin pop-up window */
		definitionDiv.scrollTop = 0;
		definitionDiv.scrollLeft = 0;


		/* position the dictionary plugin pop-up window */
		outerDiv.style.left = (window.multidict_lookup.clientX)+"px";
		outerDiv.style.top = (window.multidict_lookup.clientY)+"px";

		/* make visible the dictionary plugin pop-up window */
		outerDiv.style.display='block';
		definitionDiv.style.display='block';
	},

	/**
	 * Show the word defintion in the dictionary plugin pop-up window.
	 *
	 * @return void
	 **/
	onDefinitionLoad: function(aEvent) {
		/* the dictionary URL requested earlier has now loaded */
		window.multidict_lookup.doc = aEvent.originalTarget;
		var url = window.multidict_lookup.doc.location.href;

		if (window.multidict_lookup.doc.nodeName == "#document") {
			var outerDiv=content.document.getElementById('tfd');
			var definitionDiv=content.document.getElementById('tfd-lookup');
		};

		/* get notified when the user clicks outside the dictionary plugin pop-up window */
		content.document.addEventListener("keydown", window.multidict_lookup.tfdKeyPress,false);
		content.document.addEventListener("click", window.multidict_lookup.tfdKeyPress,false);

		/* parse the dictionary page to get just the relevant info needed */
		var tbl=window.multidict_lookup.doc.querySelector(window.multidict_lookup["dictionaries"][window.multidict_lookup.dict].selector);
		//window.multidict_lookup.jsdump(tbl);

		/* remove the loading animation */
		while (definitionDiv.firstChild) {
			definitionDiv.removeChild(definitionDiv.firstChild);
		}

		/* check if a definition was retrieved */
		if(!tbl) {

			/* use localised strings */
			tbl = content.document.createTextNode(stringsBundle.getString('multidict_lookup.popupWindow.errorDefNotFound'));
			//tbl = content.document.createTextNode('No definition found.');
		}

		/* add the definition */
		definitionDiv.appendChild(tbl);
	},


	/**
	 * Hide the dictionary plugin pop-up window when the user clicks outside.
	 *
	 * @return void
	 **/
	tfdKeyPress:function(aEvent){
		content.document.getElementById("tfd").style.display = 'none';
		content.document.removeEventListener("keydown", window.multidict_lookup.tfdKeyPress, false);
		content.document.removeEventListener("click", window.multidict_lookup.tfdKeyPress, false);
	},

	jsdump: function(str) {
		Components.classes['@mozilla.org/consoleservice;1']
            .getService(Components.interfaces.nsIConsoleService)
            .logStringMessage(str);
	}

};
window.addEventListener("load", window.multidict_lookup.onLoad, false);
